//***********************************************************************************************
// Hardwarebasis:   F4DISCOVERY
// Programm:        Blinky mit Systick
//
// Die rote und blaue LED blinkt auf dem STM32F4Discovery-Board im 1/2 Sekundentakt.
//
// Mikrocontroller: STM32F407VG (ARM 32-bit CORTEX-M4)
// Speicher:        1MB FLASH, 192KB SRAM
// Taktfrequenz:    Quarz: 8MHz, mit PLL auf: 168MHz
// Stand:           13.04.2012
// IDE/Compiler:    CooCox CoIDE 1.4.1
// STM FW-Lib:      V 1.0.0 RC1
//***********************************************************************************************

/* Includes */
#include <stddef.h>
#include "stm32f4xx.h"

/*
  Achtung!!
  HSE_VALUE = 8000000 als globales Symbol setzen (Quarzfrequenz 8MHz)
  Defaultwert ist 25MHz, sonst stimmt die PLL nicht.)
  Systemtakt wird dann automatisch auf 168MHz gesetzt.

  Alternativ: HSE_VALUE in stm32f4xx.h auf 8000000 setzen

  und:

  in system_stm32f4xx.c mssen folgende Einstellungen fr die PLL von 8Mhz auf 168MHz stehen:

  #define PLL_M      8
  #define PLL_N      336

*/

//***********************************************************************************************
// Defines
//***********************************************************************************************
#define LD_3_AUS      GPIO_ResetBits(GPIOD,GPIO_Pin_13) /* orange LED */
#define LD_3_EIN      GPIO_SetBits(GPIOD,GPIO_Pin_13)
#define LD_4_AUS      GPIO_ResetBits(GPIOD,GPIO_Pin_12) /* grne LED  */
#define LD_4_EIN      GPIO_SetBits(GPIOD,GPIO_Pin_12)
#define LD_5_AUS      GPIO_ResetBits(GPIOD,GPIO_Pin_14) /* rot LED    */
#define LD_5_EIN      GPIO_SetBits(GPIOD,GPIO_Pin_14)
#define LD_6_AUS      GPIO_ResetBits(GPIOD,GPIO_Pin_15) /* blaue LED  */
#define LD_6_EIN      GPIO_SetBits(GPIOD,GPIO_Pin_15)

#define PB_15_L       GPIO_ResetBits(GPIOB,GPIO_Pin_15) /* Test-Pin PB15  */
#define PB_15_H       GPIO_SetBits(GPIOB,GPIO_Pin_15)

//***********************************************************************************************
// Variablen
//***********************************************************************************************
volatile uint8_t blink_counter   = 1;
volatile uint8_t blink_pol       = 0;
volatile uint8_t blink_enable    = 0;

//***********************************************************************************************
// SysTick Funktion / Wiederholrate: 10ms
// siehe stm32f10x_it.c
//***********************************************************************************************
void SysTickFunction (void) {
  // Blinkfunktion (Sekundentakt)
  if (blink_enable) {
    blink_counter--;
    if (!blink_counter) {
      blink_pol = 1 - blink_pol;
      blink_counter = 50;
    }
    if (blink_pol) {
      LD_5_EIN;
      LD_6_AUS;
    } else {
      LD_5_AUS;
      LD_6_EIN;
    }
  }
}

//***********************************************************************************************
// System-Einstellungen
//***********************************************************************************************
void SetupSystem (void) {
	RCC_ClocksTypeDef RCC_Clocks;

	// SysTick aktivieren
    // 10ms Wiederholrate
	RCC_GetClocksFreq(&RCC_Clocks);
	SysTick_Config(RCC_Clocks.HCLK_Frequency / 100);
}

//***********************************************************************************************
// Peripherie-Einstellungen
//***********************************************************************************************
void SetupPeripherie (void) {
  GPIO_InitTypeDef  GPIO_InitStructure;

  // GPIOD Takt freigeben
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD, ENABLE);
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOB, ENABLE);

  // LED-GPIO als Ausgang konfigurieren
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_Init(GPIOD, &GPIO_InitStructure);

  // PB15 als Ausgang konfigurieren
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_15;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_Init(GPIOB, &GPIO_InitStructure);

}

//***********************************************************************************************
// Hauptprogramm
//***********************************************************************************************

// SystemInit(); wird automatisch vor der MAIN-Funktion
// vom startup_stm32f4xx.s gerufen

int main (void) {
  SetupSystem();        // System-Einstellungen
  SetupPeripherie();    // Peripherie-Einstellungen
  SystemCoreClock = SystemCoreClock;
  LD_3_EIN;
  LD_4_EIN;
  LD_5_AUS;
  LD_6_AUS;
  blink_enable = 1;
  while(1) {
	  GPIOB->BSRRH = GPIO_Pin_15; //PB_15_L;
	  GPIOB->BSRRL = GPIO_Pin_15; //PB_15_H;
  }
}

#ifdef  USE_FULL_ASSERT

/**
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t* file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {
  }
}
#endif

/*
 * Minimal __assert_func used by the assert() macro
 * */
void __assert_func(const char *file, int line, const char *func, const char *failedexpr)
{
  while(1)
  {}
}

/*
 * Minimal __assert() uses __assert__func()
 * */
void __assert(const char *file, int line, const char *failedexpr)
{
   __assert_func (file, line, NULL, failedexpr);
}

#ifdef USE_SEE
#ifndef USE_DEFAULT_TIMEOUT_CALLBACK
/**
  * @brief  Basic management of the timeout situation.
  * @param  None.
  * @retval sEE_FAIL.
  */
uint32_t sEE_TIMEOUT_UserCallback(void)
{
  /* Return with error code */
  return sEE_FAIL;
}
#endif
#endif /* USE_SEE */


